#!/usr/bin/python
#-*- coding:utf-8 -*-
#*******************************************************************
#  周波数特性アナライザ　データ収集しグラフを作成する
#　-周波数を波形生成ボードとレベル測定ボードにに送信　
#  -レベル測定ボードからレベル値を受信しグラフを作成し保存する
#  -毎回使用したグラフ関連リソースを解放する必要がある
#******************************************************************
from time import sleep
import subprocess as proc
import matplotlib as mplt
mplt.use("Agg")
import matplotlib.pyplot as plt
import serial
import math

# ディレクトリ指定とファイル名指定
path = '/home/pi/Analyzer/'
graphfile = 'graph1.png'
filename1 = 'Freq.txt'
filename2 = 'Endflag.txt'

# 変数定義
EndFlag = 0
sFreq = 100
eFreq = 1000
fStep = 0
tStep = 100
LevelA = []		# グラフデータ格納リスト
LevelB = []
Frequency = []

#**************** グラフ生成関数 ********************
def makeGraph():
	# グラフのインスタンス取得
	fig, ax1 = plt.subplots(1,1,figsize=(12, 6))
	# Y軸の目盛表示
	plt.ylim([-60.0, 20.0])           #Y軸スケール指定
	ax1.set_ylabel('dB', color='g')
	# 横軸の追加
	ax1.axhline(y=-50.0, color='gray', linestyle=':')
	ax1.axhline(y=-40.0, color='gray', linestyle=':')
	ax1.axhline(y=-30.0, color='gray', linestyle=':')
	ax1.axhline(y=-20.0, color='gray', linestyle=':')
	ax1.axhline(y=-10.0, color='gray', linestyle=':')
	ax1.axhline(y= 0.0,  color='gray', linestyle='-')
	ax1.axhline(y= 10.0, color='gray', linestyle=':')
	plt.xlim([0,2580])			#X軸スケール指定
	#縦軸の表示　対数表示
	for i in range(0, 7):
		for j in range(1, 11):
			ax1.axvline(x=(int)(math.log10(j*math.pow(10, i))*400), color='gray', linestyle=':')
	for i in range(1, 3):
		ax1.axvline(x=(int)(math.log10(8*math.pow(10, i))*400), color='gray', linestyle=':')
	#X軸目盛表示とスケール表示　対数周波数表示
	plt.xticks([0,400,800,1200,1600,2000,2400],[10,100,'1k','10k','100k','1M','10M'])
	ax1.set_xlabel('Frequency', color='g') 	
	#ラベル類表示、日本語の表示
	fontprop=mplt.font_manager.FontProperties(fname="/usr/share/fonts/truetype/sazanami/sazanami-gothic.ttf")
	#日本語タイトル
	ax1.set_title(u'周波数特性', fontsize=18, fontproperties=fontprop)
	#グラフ描画
	ax1.plot(Frequency, LevelA, 'r')	#チャネルA赤色
	ax1.plot(Frequency, LevelB, 'b')	#チャネルB青色
	#グラフデータファイル保存
	fig.savefig(path+graphfile)		#グラフのファイル保存
	plt.close('all')				#matoplotのリソース解放

#*************** メイン関数　**********************
def main():
	global sFreq, eFreq, fStep, tStep, LevelA, LevelB, EndFlag, Frequency

	#シリアルインターフェース定義
	conA=serial.Serial('/dev/ttyAMA0', 115200, timeout=2.0)	#レベル測定ボード
	conG=serial.Serial('/dev/ttyUSB0', 115200, timeout=2.0)	#波形生成ボード
	#コマンド取得
	f = open(path+filename1, 'r')
	Header = f.read(2)		#SN
	sFreq = long(f.read(8))	#開始周波数
	eFreq = long(f.read(8))	#終了周波数
	fStep = int(f.read(1))	#周波数ステップ
	tStep = int(f.read(4))	#時間間隔（未使用）
	f.close()

	#************* データ取得とグラフ作成 ******************
	try:
		while((sFreq < eFreq+100)and(EndFlag != '1')):
			#*** 途中停止要求チェック ***
			f = open(path+filename2, 'r')
			EndFlag = f.read(1)
			f.close()
			#周波数出力要求コマンド送信　波形生成ボードへ
			temp = '00000000' + str(sFreq)
			temp = temp[len(temp)-8:len(temp)]
			conG.write('SN'+temp+'00000E')
			#レベル計測要求コマンド送信　レベル測定ボードへ
			temp = '00000000' + str(sFreq)
			temp = temp[len(temp)-8:len(temp)]
			conA.write('SN'+temp+'E')
			#計測データ受信実行　レベル測定ボードより
			data = conA.read(16)
#			print data
			if((data[0]=='S') and (data[1]=='L')):
				LevelA.append((float)(data[2:7]))
				LevelB.append((float)(data[9:14]))
				Frequency.append((math.log10(sFreq)-1)*400)
				#グラフ作成実行
				makeGraph()
				#ファイルをコピーしファイル名変更　graph2の方を表示に使う
				proc.call("sudo cp /home/pi/Analyzer/graph1.png /home/pi/Analyzer/graph2.png",shell=True) 
			if(fStep == 0):
				sFreq += (long)(math.pow(10, (int)(math.log10(sFreq))))
			elif(fStep == 1):
				sFreq += (long)(math.pow(10, (int)(math.log10(sFreq)-1)))
			else:
				sFreq += (long)(math.pow(10, (int)(math.log10(sFreq)-2)))
#			sleep(tStep/1000)
				

	except Exception as e:
		print e.message
	print "End"

#************* 起動確認 *****************
if __name__=='__main__':
	main()